/** @odoo-module **/

import { registry } from "@web/core/registry";
import { _lt } from "@web/core/l10n/translation";
import { useInputField } from "@web/views/fields/input_field_hook";
import { standardFieldProps } from "@web/views/fields/standard_field_props";

import { Component } from "@odoo/owl";

export class SearchField extends Component {
  setup() {
    useInputField({ getValue: () => this.props.record.data[this.props.name] || "" });
    this.action = null;
  }

  async search() {
    this.action = await this.env.services.orm.call(
      "res.partner",
      "get_legal_entity_data",
      [this.env.model.root.data.id],
      {
        vat: this.props.record.data[this.props.name],
      }
    );
    await this.env.services.action.doAction(this.action, {
      onClose: async (closeInfo) => {
        if (closeInfo && closeInfo.update) {
          // Creates record
          const record = this.props.record;
          const { management, ...newRecordData } = this.action.context;
          await record.update({
            ...newRecordData,
            company_type: "company",
          });
          await record.save();

          const recordChildren = record.data.child_ids.records;
          if (
            management &&
            !this._checkManagerExists(recordChildren, management)
          ) {
            await this._createManager(management);
          }

          await this.env.model.load({
            resId: record._config.resId,
            mode: record.mode,
          });


          this.env.services.notification.add(_lt("Data updated."), {
            type: "info",
          });
        }
      },
    });
  }

  _checkManagerExists(recordChildren, management) {
    const managerName = management.manager_name;
    const managerFunction = management.manager_position;
    for (let record of recordChildren) {
      if (
        record.data.name.toUpperCase() === managerName.toUpperCase() &&
        record.data.function.toUpperCase() === managerFunction.toUpperCase()
      )
        return true;
    }
    return null;
  }

  async _createManager(management) {
    const record = this.props.record;
    const point = this.env.model;

    const paren_id = record._config.resId
    const params = {
      resModel: "res.partner",
      resIds: [],
      fields: record.data.child_ids.fields,
      activeFields: record.data.child_ids.activeFields,
      context: {
        default_name: management.manager_name,
        default_function: management.manager_position,
        default_parent_id: paren_id,
        default_type: "contact",
      },
      mode: "edit",
      viewType: "form",
      isMonoRecord: true,
      groupBy: [],
    };
    const newRecordData = await point._loadNewRecord(params);
    const newRecord = await point._createRoot(params, newRecordData);
    await newRecord.save();
  }
}



SearchField.template = "dadata_connector.SearchField";
SearchField.props = {
  ...standardFieldProps,
  placeholder: { type: String, optional: true },
};

SearchField.displayName = _lt("Search");
SearchField.supportedTypes = ["char"];

SearchField.extractProps = ({ attrs }) => {
  return {
    placeholder: attrs.placeholder,
  };
};

class FormSearchField extends SearchField { }
FormSearchField.template = "dadata_connector.FormSearchField";

registry.category("fields").add("search", { component: SearchField });
registry.category("fields").add("form.search", { component: FormSearchField });

